%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                                   genFBMJFC
%
%  function generating a time series from a fractional Gaussian noise (and the 
%  associated fractional Brownian motion) from the circulant matrix procedure
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Input	    	n    : length of the time series (positive integer)
%    		      H    : self-similarity parameter (0<H<1)
%
%  Output	        fGN  : a vector (fGn(1),..,fGn(n)) of a path of a fractional 
%                          Gaussian noise of parameter H and variance 1
%                   fBm  : a vector (fBm(1),..,fBm(n)) of a path of a fractional
% 			               Brownian motion such that fGn(i)=fBm(i+1)-fBm(i) for 
%                          i=1,2,..,n
%
%  Example          [fGn,fBm]=genFBMJFC(1000,0.3)
%
%		Created by J-F. Coeurjolly (Universit de Grenoble, France) + Bardet, 2008
%                          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [fGn,fBm]=genFBMJFC(n,H)

m=2^nextpow2(2*(n-1));	    	% research of the power of two greater than 2*(n-1)
u=covFGNJFC(m/2,H); 	        % u=c(0),c(1),...c(m/2)
ligneC=[u fliplr(u(2:(m/2-1)))];    % [[c(0),c(1),...c(m/2)],[c(m/2-1),...,c(1)]]
eg=real(fft(ligneC));           % eigenvalues of the Circulant Matrix 
m  		=  length(eg);
ar		=  randn(1,fix(m/2 + 1));  	
ai		=  randn(1,fix(m/2 + 1));			
ar(1) 		=  sqrt(2) * ar(1);
ar(fix(m/2 + 1)) 	=  sqrt(2) * ar(fix(m/2 + 1));
ai(1) 	=  0;
ai(fix(m/2 + 1)) 	=  0;
ar		= [ar(1:fix(m/2 + 1))  ar(fix(m/2):-1:2)];
aic 	= - ai;
ai		= [ai(1:fix(m/2 + 1))  aic(fix(m/2):-1:2)];
a		= (1/(sqrt(2 * m))) *real(fft(sqrt(eg) .*(ar+i*ai))); % simulation of a fGn 
                                                              % of length m
fGn     = a(1:n);               % simulation of a fGn of length n 
fBm		= cumsum(fGn);          % simulation of a fBm

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function cFGN=covFGNJFC(n,H)

k =  0:(n - 1);
H2 = 2 * H;
cFGN = (abs((k - 1)/n).^H2 - 2 * abs(k/n).^H2 + abs((k + 1)/n).^H2)/2;
